@@ -1,6 +1,6 @@
 package Class::Slot;
 # ABSTRACT: Simple, efficient, comple-time class declaration
-$Class::Slot::VERSION = '0.07';
+$Class::Slot::VERSION = '0.08';
 use strict;
 use warnings;
 
@@ -32,54 +32,37 @@ BEGIN {
 
 INIT {
   $LATE = 1;
+
+  # When multiple packages are defined in a single file or top-level string
+  # eval, they will generate a definition before INIT is called. If they refer
+  # to each other, one may call a method of the other before the class' init
+  # has been called.
+  #
+  # To handle this case, we scan the %CLASS definitions for classes which have
+  # been defined but not yet initialized - that is, they are in %CLASS but the
+  # 'init' function hasn't been run yet (it deletes itself when it completes) -
+  # and then run those classes' initializers..
+  for my $class (keys %CLASS) {
+    next unless exists $Class::Slot::CLASS{$class}{init};
+
+    *{$class. '::new'} = sub {
+      $Class::Slot::CLASS{$class}{init}->();
+      goto $class->can('new');
+    };
+  }
 }
 
 sub import {
   my $class = shift;
-  my $name  = shift || return;
+  my $name  = shift;
   my ($caller, $file, $line) = caller;
 
-  # Handle special parameters
-  if ($name eq '-debugall') {
-    $DEBUG_ALL = 1;
-    return;
-  }
-
-  if ($name eq '-debug') {
-    $DEBUG{$caller} = 1;
-    return;
-  }
-
-  if ($name =~ /^c3$/i) {
-    $C3{$caller} = 1;
-    return;
-  }
-
-  # Suss out slot parameters
-  my ($type, %param) = (@_ % 2 == 0)
-    ? (undef, @_)
-    : @_;
-
-  $type = Class::Slot::AnonType->new($type)
-    if ref $type eq 'CODE';
-
-  croak "slot ${name}'s type is invalid"
-    if defined $type
-    && !ref $type
-    && !$type->can('can_be_inlined')
-    && !$type->can('inline_check')
-    && !$type->can('check');
-
-  # Ensure that the default value is valid if the type is set
-  if (exists $param{def} && $type) {
-    croak "default value for $name is not a valid $type"
-      unless $type->check(ref $param{def} eq 'CODE' ? $param{def}->() : $param{def});
-  }
-
-  # Initialize slot storage
+  # Initialize the class
   unless (exists $CLASS{$caller}) {
     $C3{$caller} ||= 0;
 
+    *{ $caller . '::get_slots' } = \&get_slots;
+
     $CLASS{$caller} = {
       slot  => {}, # slot definitions
       slots => [], # list of slot names
@@ -99,15 +82,22 @@ sub import {
           }
         }
 
+        my %slots = %{ $caller->get_slots };
+
         # Build constructor
         my $ctor = _build_ctor($caller);
 
         # Build accessors
-        my $acc = join "\n", map{ _build_accessor($caller, $_) } keys %{ $caller->get_slots };
+        my $acc = join "\n", map{ _build_accessor($caller, $_) }
+          keys %slots;
+
+        # Build delegate accessors
+        my $delegates = join "\n", map{ _build_delegates($caller, $_) }
+          keys %slots;
 
         # Build @SLOTS
         my $slots = join ' ', map{ quote_identifier($_) }
-          sort keys %{ $caller->get_slots };
+          sort keys %slots;
 
         my $pkg  = qq{package $caller;
 no warnings 'redefine';
@@ -124,7 +114,12 @@ $ctor
 #-------------------------------------------------------------------------------
 # Accessors
 #-------------------------------------------------------------------------------
-$acc};
+$acc
+
+#-------------------------------------------------------------------------------
+# Delegate accessors
+#-------------------------------------------------------------------------------
+$delegates};
 
         if ($DEBUG_ALL || $DEBUG{$caller}) {
           print "\n";
@@ -170,26 +165,75 @@ CHECK {
     }
   }
 
-  $CLASS{$caller}{slot}{$name} = {
-    pkg  => $caller,
-    file => $file,
-    line => $line,
-  };
+  if (defined $name) {
+    # Handle special parameters
+    if ($name eq '-debugall') {
+      $DEBUG_ALL = 1;
+      return;
+    }
 
-  if (defined $type) {
-    my $addr = refaddr $type;
-    $CLASS{$caller}{slot}{$name}{type} = $addr;
-    $TYPE{$addr} = $type;
+    if ($name eq '-debug') {
+      $DEBUG{$caller} = 1;
+      return;
+    }
+
+    if ($name =~ /^c3$/i) {
+      $C3{$caller} = 1;
+      return;
+    }
+
+    # Suss out slot parameters
+    my ($type, %param) = (@_ % 2 == 0)
+      ? (undef, @_)
+      : @_;
+
+    $type = Class::Slot::AnonType->new($type)
+      if ref $type eq 'CODE';
+
+    croak "slot ${name}'s type is invalid"
+      if defined $type
+      && !ref $type
+      && !$type->can('can_be_inlined')
+      && !$type->can('inline_check')
+      && !$type->can('check');
+
+    # Ensure that the default value is valid if the type is set
+    if (exists $param{def} && $type) {
+      croak "default value for $name is not a valid $type"
+        unless $type->check(ref $param{def} eq 'CODE' ? $param{def}->() : $param{def});
+    }
+
+    # Validate that delegate methods are defined as an array or hash ref
+    if (exists $param{fwd}) {
+      croak "delegate forwarding for $name must be expressed as an array ref or hash ref"
+        if ref($param{fwd}) !~ /^(?:ARRAY)|(?:HASH)$/;
+
+      if (ref $param{fwd} eq 'ARRAY') {
+        my %tmp;
+        $tmp{$_} = $_ for @{$param{fwd}};
+        $param{fwd} = \%tmp;
+      }
+    }
+
+    $CLASS{$caller}{slot}{$name} = {
+      pkg  => $caller,
+      file => $file,
+      line => $line,
+    };
+
+    if (defined $type) {
+      my $addr = refaddr $type;
+      $CLASS{$caller}{slot}{$name}{type} = $addr;
+      $TYPE{$addr} = $type;
+    }
+
+    for (qw(def req rw fwd)) {
+      $CLASS{$caller}{slot}{$name}{$_} = $param{$_}
+        if exists $param{$_};
+    }
+
+    push @{ $CLASS{$caller}{slots} }, $name;
   }
-
-  for (qw(def req rw)) {
-    $CLASS{$caller}{slot}{$name}{$_} = $param{$_}
-      if exists $param{$_};
-  }
-
-  *{ $caller . '::get_slots' } = \&get_slots;
-
-  push @{ $CLASS{$caller}{slots} }, $name;
 }
 
 #-------------------------------------------------------------------------------
@@ -326,6 +370,28 @@ sub get_slots {
   }
 }
 
+#-------------------------------------------------------------------------------
+# Delegate methods
+#-------------------------------------------------------------------------------
+sub _build_delegates {
+  my ($class, $name) = @_;
+  my $slot = $class->get_slots($name);
+  return '' unless exists $slot->{fwd};
+
+  my $fwd   = $slot->{fwd};
+  my $line  = qq{# line $slot->{line} "$slot->{file}"};
+  my $ident = quote_identifier($name);
+  my $code  = '';
+
+  for (keys %$fwd) {
+    my $local_method  = quote_identifier($_);
+    my $remote_method = quote_identifier($fwd->{$_});
+    $code .= "$line\nsub $local_method { shift->${ident}->${remote_method}(\@_) }";
+  }
+
+  return $code;
+}
+
 #-------------------------------------------------------------------------------
 # Accessors
 #-------------------------------------------------------------------------------
@@ -463,7 +529,7 @@ FILTER {
 
 
 package Class::Slot::AnonType;
-$Class::Slot::AnonType::VERSION = '0.07';
+$Class::Slot::AnonType::VERSION = '0.08';
 use strict;
 use warnings;
 use Carp;
@@ -498,7 +564,7 @@ Class::Slot - Simple, efficient, comple-time class declaration
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -613,6 +679,51 @@ If the default is a code ref which generates a value and a type is specified,
 note that the code ref will be called during compilation to validate its type
 rather than re-validating it with every accessor call.
 
+=head2 fwd
+
+When present, generates delegate accessor methods that forward to a mapped
+method on the object stored in the slot. For example:
+
+  # Foo.pm
+  class Foo;
+
+  sub life{ 42 }
+
+  1;
+
+
+  # Bar.pm
+  class Bar;
+  use Class::Slot;
+  use parent 'Foo';
+
+  slot 'foo', fwd => ['life'];
+
+  1;
+
+
+  # main.pl
+  my $bar = Bar->new(foo => Foo->new);
+  say $bar->life; # calls $bar->foo->life
+
+Alternately, C<fwd> may be defined as a hash ref mapping new local method
+names to method names in the delegate class:
+
+  # Bar.pm
+  class Bar;
+  use Class::Slot;
+  use parent 'Foo';
+
+  slot 'foo', fwd => {barlife => 'life'};
+
+  1;
+
+
+  # main.pl
+  my $bar = Bar->new(foo => Foo->new);
+  say $bar->barlife; # calls $bar->foo->life
+  say $bar->life;    # dies: method not found
+
 =head1 INHERITANCE
 
 When a class declares a slot which is also declared in the parent class, the
